@@ -1,31 +1,27 @@
 #! /usr/local/bin/ruby
 # -*- mode: ruby; coding: us-ascii -*-
+# frozen_string_literal: false
 
 # :stopdoc:
 $extension = nil
 $extstatic = nil
 $force_static = nil
-$install = nil
 $destdir = nil
 $dryrun = false
-$clean = nil
 $nodynamic = nil
-$extinit = nil
 $extobjs = []
 $extflags = ""
 $extlibs = nil
 $extpath = nil
-$ignore = nil
 $message = nil
 $command_output = nil
-$configure_only = false
+$subconfigure = false
 
 $progname = $0
 alias $PROGRAM_NAME $0
 alias $0 $progname
 
 $extlist = []
-$compiled = {}
 
 DUMMY_SIGNATURE = "***DUMMY MAKEFILE***"
 
@@ -49,10 +45,6 @@ elsif !File.chardev?(@null = "/dev/null")
   @null = "nul"
 end
 
-def sysquote(x)
-  @quote ? x.quote : x
-end
-
 def verbose?
   $mflags.defined?("V") == "1"
 end
@@ -84,11 +76,12 @@ def extract_makefile(makefile, keep = true)
   m = File.read(makefile)
   s = m[/^CLEANFILES[ \t]*=[ \t](.*)/, 1] and $cleanfiles = s.split
   s = m[/^DISTCLEANFILES[ \t]*=[ \t](.*)/, 1] and $distcleanfiles = s.split
+  s = m[/^EXTSO[ \t]*=[ \t](.*)/, 1] and $extso = s.split
   if !(target = m[/^TARGET[ \t]*=[ \t]*(\S*)/, 1])
     return keep
   end
   installrb = {}
-  m.scan(/^install-rb-default:.*[ \t](\S+)(?:[ \t].*)?\n\1:[ \t]*(\S+)/) {installrb[$2] = $1}
+  m.scan(/^(?:do-)?install-rb-default:.*[ \t](\S+)(?:[ \t].*)?\n\1:[ \t]*(\S+)/) {installrb[$2] = $1}
   oldrb = installrb.keys.sort
   newrb = install_rb(nil, "").collect {|d, *f| f}.flatten.sort
   if target_prefix = m[/^target_prefix[ \t]*=[ \t]*\/(.*)/, 1]
@@ -109,7 +102,7 @@ def extract_makefile(makefile, keep = true)
   srcs = Dir[File.join($srcdir, "*.{#{SRC_EXT.join(%q{,})}}")].map {|fn| File.basename(fn)}.sort
   if !srcs.empty?
     old_srcs = m[/^ORIG_SRCS[ \t]*=[ \t](.*)/, 1] or return false
-    old_srcs.split.sort == srcs or return false
+    (old_srcs.split - srcs).empty? or return false
   end
   $target = target
   $extconf_h = m[/^RUBY_EXTCONF_H[ \t]*=[ \t]*(\S+)/, 1]
@@ -128,24 +121,20 @@ def extract_makefile(makefile, keep = true)
   end
   $objs = (m[/^OBJS[ \t]*=[ \t](.*)/, 1] || "").split
   $srcs = (m[/^SRCS[ \t]*=[ \t](.*)/, 1] || "").split
+  $headers = (m[/^LOCAL_HDRS[ \t]*=[ \t](.*)/, 1] || "").split
   $LOCAL_LIBS = m[/^LOCAL_LIBS[ \t]*=[ \t]*(.*)/, 1] || ""
   $LIBPATH = Shellwords.shellwords(m[/^libpath[ \t]*=[ \t]*(.*)/, 1] || "") - %w[$(libdir) $(topdir)]
   true
 end
 
-def extmake(target)
-  unless $configure_only || verbose?
-    print "#{$message} #{target}\n"
-    $stdout.flush
-  end
-
+def extmake(target, basedir = 'ext', maybestatic = true)
   FileUtils.mkpath target unless File.directory?(target)
   begin
     # don't build if parent library isn't build
     parent = true
     d = target
     until (d = File.dirname(d)) == '.'
-      if File.exist?("#{$top_srcdir}/ext/#{d}/extconf.rb")
+      if File.exist?("#{$top_srcdir}/#{basedir}/#{d}/extconf.rb")
         parent = (/^all:\s*install/ =~ IO.read("#{d}/Makefile") rescue false)
         break
       end
@@ -163,16 +152,16 @@ def extmake(target)
     $topdir = prefix + $topdir
     $target = target
     $mdir = target
-    $srcdir = File.join($top_srcdir, "ext", $mdir)
+    $srcdir = File.join($top_srcdir, basedir, $mdir)
     $preload = nil
     $objs = []
     $srcs = []
-    $compiled[target] = false
+    $extso = []
     makefile = "./Makefile"
     static = $static
     $static = nil if noinstall = File.fnmatch?("-*", target)
     ok = parent && File.exist?(makefile)
-    if parent && !$ignore
+    if parent
       rbconfig0 = RbConfig::CONFIG
       mkconfig0 = CONFIG
       rbconfig = {
@@ -182,7 +171,7 @@ def extmake(target)
       }
       mkconfig = {
 	"hdrdir" => ($hdrdir == top_srcdir) ? top_srcdir : "$(top_srcdir)/include",
-	"srcdir" => "$(top_srcdir)/ext/#{$mdir}",
+	"srcdir" => "$(top_srcdir)/#{basedir}/#{$mdir}",
 	"topdir" => $topdir,
       }
       rbconfig0.each_pair {|key, val| rbconfig[key] ||= val.dup}
@@ -208,14 +197,12 @@ def extmake(target)
 	    [conf, "#{$srcdir}/depend"].any? {|f| modified?(f, [t])})
         then
 	  ok = false
-          if $configure_only
-            if verbose?
-              print "#{conf}\n" if conf
-            else
-              print "#{$message} #{target}\n"
-            end
-            $stdout.flush
+          if verbose?
+            print "#{conf}\n" if conf
+          else
+            print "#{$message} #{target}\n"
           end
+          $stdout.flush
           init_mkmf
 	  Logging::logfile 'mkmf.log'
 	  rm_f makefile
@@ -242,45 +229,35 @@ def extmake(target)
       end
     end
     ok &&= File.open(makefile){|f| s = f.gets and !s[DUMMY_SIGNATURE]}
-    ok = yield(ok) if block_given?
     unless ok
+      mf = ["# #{DUMMY_SIGNATURE}\n", *dummy_makefile(CONFIG["srcdir"])].join("")
       atomic_write_open(makefile) do |f|
-        f.puts "# " + DUMMY_SIGNATURE
-	f.print(*dummy_makefile(CONFIG["srcdir"]))
+        f.print(mf)
       end
 
       return true if !error and target.start_with?("-")
 
-      mess = "Failed to configure #{target}. It will not be installed.\n"
+      message = nil
       if error
-        mess = "#{error}\n#{mess}"
+        loc = error.backtrace_locations[0]
+        message = "#{loc.absolute_path}:#{loc.lineno}: #{error.message}"
+        if Logging.log_opened?
+          Logging::message("#{message}\n\t#{error.backtrace.join("\n\t")}\n")
+        end
       end
 
-      Logging::message(mess) if Logging.log_opened?
-      print(mess)
-      $stdout.flush
-      return true
+      return [parent, message]
     end
-    args = sysquote($mflags)
+    args = $mflags
     unless $destdir.to_s.empty? or $mflags.defined?("DESTDIR")
-      args += [sysquote("DESTDIR=" + relative_from($destdir, "../"+prefix))]
+      args += ["DESTDIR=" + relative_from($destdir, "../"+prefix)]
     end
     if $static and ok and !$objs.empty? and !noinstall
-      args += ["static"] unless $clean
-      $extlist.push [$static, target, $target, $preload]
+      args += ["static"]
+      $extlist.push [(maybestatic ? $static : false), target, $target, $preload]
     end
     FileUtils.rm_f(old_cleanfiles - $distcleanfiles - $cleanfiles)
     FileUtils.rm_f(old_objs - $objs)
-    unless $configure_only or system($make, *args)
-      $ignore or $continue or return false
-    end
-    $compiled[target] = true
-    if $clean
-      FileUtils.rm_f("mkmf.log")
-      if $clean != true
-	FileUtils.rm_f([makefile, $extconf_h || "extconf.h"])
-      end
-    end
     if $static
       $extflags ||= ""
       $extlibs ||= []
@@ -322,10 +299,6 @@ def extmake(target)
   true
 end
 
-def compiled?(target)
-  $compiled[target]
-end
-
 def parse_args()
   $mflags = []
   $makeflags = [] # for make command to build ruby, so quoted
@@ -380,6 +353,7 @@ def parse_args()
     $optparser.warn(e)
     abort $optparser.to_s
   end
+  $command_output or abort "--command-output option is mandatory"
 
   $destdir ||= ''
 
@@ -415,18 +389,10 @@ parse_args()
 if target = ARGV.shift and /^[a-z-]+$/ =~ target
   $mflags.push(target)
   case target
-  when /^(dist|real)?(clean)$/
-    target = $2
-    $ignore ||= true
-    $clean = $1 ? $1[0] : true
-  when /^install\b/
-    $install = true
-    $ignore ||= true
-    $mflags.unshift("INSTALL_PROG=install -c -p -m 0755",
-                    "INSTALL_DATA=install -c -p -m 0644",
-                    "MAKEDIRS=mkdir -p") if $dryrun
+  when /^(dist|real)?(clean)$/, /^install\b/
+    abort "#{target} is obsolete"
   when /configure/
-    $configure_only = true
+    $subconfigure = !ARGV.empty?
   end
 end
 unless $message
@@ -445,12 +411,15 @@ elsif sep = config_string('BUILD_FILE_SEPARATOR')
 else
   $ruby = '$(topdir)/miniruby' + EXEEXT
 end
-$ruby << " -I'$(topdir)'"
+$ruby = [$ruby]
+$ruby << "-I'$(topdir)'"
 unless CROSS_COMPILING
-  $ruby << " -I'$(top_srcdir)/lib'"
-  $ruby << " -I'$(extout)/$(arch)' -I'$(extout)/common'" if $extout
+  $ruby << "-I'$(top_srcdir)/lib'"
+  $ruby << "-I'$(extout)/$(arch)'" << "-I'$(extout)/common'" if $extout
   ENV["RUBYLIB"] = "-"
 end
+topruby = $ruby
+$ruby = topruby.join(' ')
 $mflags << "ruby=#$ruby"
 
 MTIMES = [__FILE__, 'rbconfig.rb', srcdir+'/lib/mkmf.rb'].collect {|f| File.mtime(f)}
@@ -490,14 +459,24 @@ for dir in ["ext", File::join($top_srcdir, "ext")]
   end
 end unless $extstatic
 
-ext_prefix = "#{$top_srcdir}/ext"
+@gemname = nil
+if ARGV[0]
+  ext_prefix, exts = ARGV.shift.split('/', 2)
+  $extension = [exts] if exts
+  @gemname = exts if ext_prefix == 'gems'
+end
+ext_prefix = "#{$top_srcdir}/#{ext_prefix || 'ext'}"
 exts = $static_ext.sort_by {|t, i| i}.collect {|t, i| t}
 default_exclude_exts =
-  if $mswin or $mingw
+  case
+  when $cygwin
+    %w''
+  when $mswin, $mingw
     %w'pty syslog'
   else
     %w'*win32*'
   end
+mandatory_exts = {}
 withes, withouts = [["--with", nil], ["--without", default_exclude_exts]].collect {|w, d|
   if !(w = %w[-extensions -ext].collect {|o|arg_config(w+o)}).any?
     d ? proc {|c1| d.any?(&c1)} : proc {true}
@@ -505,46 +484,90 @@ withes, withouts = [["--with", nil], ["--without", default_exclude_exts]].collec
     proc {true}
   else
     w = w.collect {|o| o.split(/,/)}.flatten
-    w.collect! {|o| o == '+' ? d : o}.flatten! if d
+    w.collect! {|o| o == '+' ? d : o}.flatten!
     proc {|c1| w.any?(&c1)}
   end
 }
 cond = proc {|ext, *|
-  cond1 = proc {|n| File.fnmatch(n, ext)}
-  withes.call(cond1) and !withouts.call(cond1)
+  withes.call(proc {|n|
+                !n or (mandatory_exts[ext] = true if File.fnmatch(n, ext))
+              }) and
+    !withouts.call(proc {|n| File.fnmatch(n, ext)})
 }
 ($extension || %w[*]).each do |e|
   e = e.sub(/\A(?:\.\/)+/, '')
-  exts |= Dir.glob("#{ext_prefix}/#{e}/**/extconf.rb").collect {|d|
+  incl, excl = Dir.glob("#{ext_prefix}/#{e}/**/extconf.rb").collect {|d|
     d = File.dirname(d)
     d.slice!(0, ext_prefix.length + 1)
     d
-  }.find_all {|ext|
+  }.partition {|ext|
     with_config(ext, &cond)
-  }.sort
+  }
+  incl.sort!
+  excl.sort!.collect! {|d| d+"/"}
+  nil while incl.reject! {|d| excl << d+"/" if excl.any? {|x| d.start_with?(x)}}
+  exts |= incl
   if $LIBRUBYARG_SHARED.empty? and CONFIG["EXTSTATIC"] == "static"
     exts.delete_if {|d| File.fnmatch?("-*", d)}
   end
 end
+ext_prefix = File.basename(ext_prefix)
 
-if $extout
-  extout = RbConfig.expand("#{$extout}", RbConfig::CONFIG.merge("topdir"=>$topdir))
-  unless $ignore
-    FileUtils.mkpath(extout)
+extend Module.new {
+  def timestamp_file(name, target_prefix = nil)
+    if @gemname and name == '$(TARGET_SO_DIR)'
+      name = "$(arch)/gems/#{@gemname}#{target_prefix}"
+    end
+    super.sub(%r[/\.extout\.(?:-\.)?], '/.')
   end
-end
+
+  def configuration(srcdir)
+    super << "EXTSO #{['=', $extso].join(' ')}\n"
+  end
+
+  def create_makefile(*args, &block)
+    return super unless @gemname
+    super(*args) do |conf|
+      conf.find do |s|
+        s.sub!(/^(TARGET_SO_DIR *= *)\$\(RUBYARCHDIR\)/) {
+          "TARGET_GEM_DIR = $(extout)/gems/$(arch)/#{@gemname}\n"\
+          "#{$1}$(TARGET_GEM_DIR)$(target_prefix)"
+        }
+      end
+      conf.any? {|s| /^TARGET *= *\S/ =~ s} and conf << %{
+
+# default target
+all:
+
+build_complete = $(TARGET_GEM_DIR)/gem.build_complete
+install-so: build_complete
+build_complete: $(build_complete)
+$(build_complete): $(TARGET_SO)
+	$(Q) $(TOUCH) $@
+
+clean-so::
+	-$(Q)$(RM) $(build_complete)
+}
+      conf
+    end
+  end
+}
 
 dir = Dir.pwd
-FileUtils::makedirs('ext')
-Dir::chdir('ext')
+FileUtils::makedirs(ext_prefix)
+Dir::chdir(ext_prefix)
 
 hdrdir = $hdrdir
 $hdrdir = ($top_srcdir = relative_from(srcdir, $topdir = "..")) + "/include"
+extso = []
+fails = []
 exts.each do |d|
   $static = $force_static ? true : $static_ext[d]
 
-  if $ignore or !$nodynamic or $static
-    extmake(d) or abort
+  if !$nodynamic or $static
+    result = extmake(d, ext_prefix, !@gemname) or abort
+    extso |= $extso
+    fails << [d, result] unless result == true
   end
 end
 
@@ -557,36 +580,18 @@ extinit = Struct.new(:c, :o) {
     super("#{src}.c", "#{src}.#{$OBJEXT}")
   end
 }.new("extinit")
-if $ignore
-  FileUtils.rm_f(extinit.to_a) if $clean
-  Dir.chdir ".."
-  if $clean
-    Dir.rmdir('ext') rescue nil
-    if $extout
-      FileUtils.rm_rf([extout+"/common", extout+"/include/ruby", extout+"/rdoc"])
-      FileUtils.rm_rf(extout+"/"+CONFIG["arch"])
-      if $clean != true
-	FileUtils.rm_rf(extout+"/include/"+CONFIG["arch"])
-	FileUtils.rm_f($mflags.defined?("INSTALLED_LIST")||ENV["INSTALLED_LIST"]||".installed.list")
-	Dir.rmdir(extout+"/include") rescue nil
-	Dir.rmdir(extout) rescue nil
-      end
-    end
-  end
-  exit
-end
 
-$extinit ||= ""
 $extobjs ||= []
 $extpath ||= []
 $extflags ||= ""
 $extlibs ||= []
+extinits = []
 unless $extlist.empty?
-  $extinit << "\n" unless $extinit.empty?
   list = $extlist.dup
   built = []
   while e = list.shift
-    _, target, feature, required = e
+    static, target, feature, required = e
+    next unless static
     if required and !(required -= built).empty?
       l = list.size
       if (while l > 0; break true if required.include?(list[l-=1][1]) end)
@@ -595,27 +600,11 @@ unless $extlist.empty?
       next
     end
     base = File.basename(feature)
-    $extinit << "    init(Init_#{base}, \"#{feature}.so\");\n"
+    extinits << feature
     $extobjs << format("ext/%s/%s.%s", target, base, $LIBEXT)
     built << target
   end
 
-  src = %{\
-#include "ruby/ruby.h"
-
-#define init(func, name) {	\\
-    extern void func(void);	\\
-    ruby_init_ext(name, func);	\\
-}
-
-void ruby_init_ext(const char *name, void (*init)(void));
-
-void Init_ext(void)\n{\n#$extinit}
-}
-  if !modified?(extinit.c, MTIMES) || IO.read(extinit.c) != src
-    open(extinit.c, "w") {|fe| fe.print src}
-  end
-
   $extpath.delete("$(topdir)")
   $extflags = libpathflag($extpath) << " " << $extflags.strip
   conf = [
@@ -623,9 +612,9 @@ void Init_ext(void)\n{\n#$extinit}
     ['SETUP', $setup],
     ['EXTLIBS', $extlibs.join(' ')], ['EXTLDFLAGS', $extflags]
   ].map {|n, v|
-    "#{n}=#{v}" if v and !(v = v.strip).empty?
+    "#{n}=#{v}" if v &&= v[/\S(?:.*\S)?/]
   }.compact
-  puts(*conf)
+  puts(*conf) unless $subconfigure
   $stdout.flush
   $mflags.concat(conf)
   $makeflags.concat(conf)
@@ -647,22 +636,11 @@ unless $destdir.to_s.empty?
 end
 $makeflags.uniq!
 
-if $nmake == ?b
-  unless (vars = $mflags.grep(/\A\w+=/n)).empty?
-    open(mkf = "libruby.mk", "wb") do |tmf|
-      tmf.puts("!include Makefile")
-      tmf.puts
-      tmf.puts(*vars.map {|v| v.sub(/\=/, " = ")})
-      tmf.puts("PRE_LIBRUBY_UPDATE = del #{mkf}")
-    end
-    $mflags.unshift("-f#{mkf}")
-    vars.each {|flag| flag.sub!(/\A/, "-D")}
-  end
-end
 $mflags.unshift("topdir=#$topdir")
 ENV.delete("RUBYOPT")
-if $configure_only and $command_output
-  exts.map! {|d| "ext/#{d}/."}
+exts.map! {|d| "#{ext_prefix}/#{d}/."}
+FileUtils.makedirs(File.dirname($command_output))
+begin
   atomic_write_open($command_output) do |mf|
     mf.puts "V = 0"
     mf.puts "Q1 = $(V:1=)"
@@ -687,28 +665,36 @@ if $configure_only and $command_output
       puts
     end
 
+    mf.macro "ruby", topruby
+    mf.macro "RUBY", ["$(ruby)"]
     mf.macro "extensions", exts
     mf.macro "EXTOBJS", $extlist.empty? ? ["dmyext.#{$OBJEXT}"] : ["ext/extinit.#{$OBJEXT}", *$extobjs]
     mf.macro "EXTLIBS", $extlibs
+    mf.macro "EXTSO", extso
     mf.macro "EXTLDFLAGS", $extflags.split
+    mf.macro "EXTINITS", extinits
     submakeopts = []
     if enable_config("shared", $enable_shared)
-      submakeopts << 'DLDOBJS="$(EXTOBJS) $(ENCOBJS)"'
+      submakeopts << 'DLDOBJS="$(EXTOBJS) $(EXTENCS)"'
       submakeopts << 'EXTOBJS='
       submakeopts << 'EXTSOLIBS="$(EXTLIBS)"'
       submakeopts << 'LIBRUBY_SO_UPDATE=$(LIBRUBY_EXTS)'
     else
-      submakeopts << 'EXTOBJS="$(EXTOBJS) $(ENCOBJS)"'
+      submakeopts << 'EXTOBJS="$(EXTOBJS) $(EXTENCS)"'
       submakeopts << 'EXTLIBS="$(EXTLIBS)"'
     end
     submakeopts << 'EXTLDFLAGS="$(EXTLDFLAGS)"'
+    submakeopts << 'EXTINITS="$(EXTINITS)"'
     submakeopts << 'UPDATE_LIBRARIES="$(UPDATE_LIBRARIES)"'
     submakeopts << 'SHOWFLAGS='
     mf.macro "SUBMAKEOPTS", submakeopts
+    mf.macro "NOTE_MESG", %w[$(RUBY) $(top_srcdir)/tool/colorize.rb skip]
+    mf.macro "NOTE_NAME", %w[$(RUBY) $(top_srcdir)/tool/colorize.rb fail]
     mf.puts
     targets = %w[all install static install-so install-rb clean distclean realclean]
     targets.each do |tgt|
       mf.puts "#{tgt}: $(extensions:/.=/#{tgt})"
+      mf.puts "#{tgt}: note" unless /clean\z/ =~ tgt
     end
     mf.puts
     mf.puts "clean:\n\t-$(Q)$(RM) ext/extinit.#{$OBJEXT}"
@@ -716,15 +702,16 @@ if $configure_only and $command_output
     mf.puts
     mf.puts "#{rubies.join(' ')}: $(extensions:/.=/#{$force_static ? 'static' : 'all'})"
     submake = "$(Q)$(MAKE) $(MFLAGS) $(SUBMAKEOPTS)"
-    mf.puts "all static: $(EXTOBJS)\n\t#{submake} #{rubies.join(' ')}\n"
+    mf.puts "all static: #{rubies.join(' ')}\n"
     $extobjs.each do |tgt|
       mf.puts "#{tgt}: #{File.dirname(tgt)}/static"
     end
-    mf.puts "#{rubies.join(' ')}: $(EXTOBJS)"
+    mf.puts "#{rubies.join(' ')}: $(EXTOBJS)#{' libencs' if CONFIG['ENCSTATIC'] == 'static'}"
     rubies.each do |tgt|
       mf.puts "#{tgt}:\n\t#{submake} $@"
     end
-    mf.puts "ext/extinit.#{$OBJEXT}:\n\t$(Q)$(MAKE) $(MFLAGS) V=$(V) $@" if $static
+    mf.puts "libencs:\n\t$(Q)$(MAKE) -f enc.mk V=$(V) $@"
+    mf.puts "ext/extinit.#{$OBJEXT}:\n\t$(Q)$(MAKE) $(MFLAGS) V=$(V) EXTINITS=\"$(EXTINITS)\" $@" if $static
     mf.puts
     if $gnumake == "yes"
       submake = "$(MAKE) -C $(@D)"
@@ -738,32 +725,37 @@ if $configure_only and $command_output
         mf.puts "#{d[0..-2]}#{tgt}:\n\t$(Q)#{submake} $(MFLAGS) V=$(V) $(@F)"
       end
     end
-  end
-elsif $command_output
-  message = "making #{rubies.join(', ')}"
-  message = "echo #{message}"
-  $mflags.concat(rubies)
-  $makeflags.concat(rubies)
-  cmd = $makeflags.map {|ss|ss.sub(/.*[$(){};\s].*/, %q['\&'])}.join(' ')
-  open($command_output, 'wb') do |ff|
-    case $command_output
-    when /\.sh\z/
-      ff.puts message, "rm -f \"$0\"; exec \"$@\" #{cmd}"
-    when /\.bat\z/
-      ["@echo off", message, "%* #{cmd}", "del %0 & exit %ERRORLEVEL%"].each do |ss|
-        ff.print ss, "\r\n"
+    mf.puts "\n""extso:\n"
+    mf.puts "\t@echo EXTSO=$(EXTSO)"
+
+    mf.puts "\n""note:\n"
+    unless fails.empty?
+      abandon = false
+      mf.puts "note: note-body\n"
+      mf.puts "note-body:: note-header\n"
+      mf.puts "note-header:\n"
+      mf.puts %Q<\t@$(NOTE_MESG) "*** Following extensions are not compiled:">
+      mf.puts "note-body:: note-header\n"
+      fails.each do |ext, (parent, err)|
+        abandon ||= mandatory_exts[ext]
+        mf.puts %Q<\t@$(NOTE_NAME) "#{ext}:">
+        if parent
+          mf.puts %Q<\t@echo "\tCould not be configured. It will not be installed.">
+          err and err.scan(/.+/) do |ee|
+            mf.puts %Q<\t@echo "\t#{ee.gsub(/["`$^]/, '\\\\\\&')}">
+          end
+          mf.puts %Q<\t@echo "\tCheck #{ext_prefix}/#{ext}/mkmf.log for more details.">
+        else
+          mf.puts %Q<\t@echo "\tSkipped because its parent was not configured.">
+        end
+      end
+      mf.puts "note:\n"
+      mf.puts %Q<\t@$(NOTE_MESG) "*** Fix the problems, then remove these directories and try again if you want.">
+      if abandon
+        mf.puts "\t""@exit 1"
       end
-    else
-      ff.puts cmd
     end
-    ff.chmod(0755)
   end
-elsif !$configure_only
-  message = "making #{rubies.join(', ')}"
-  puts message
-  $stdout.flush
-  $mflags.concat(rubies)
-  system($make, *sysquote($mflags)) or exit($?.exitstatus)
 end
 # :startdoc:
 
