@@ -1,4 +1,5 @@
-require 'rdoc/test_case'
+# frozen_string_literal: true
+require 'minitest_helper'
 
 class TestRDocRIDriver < RDoc::TestCase
 
@@ -46,13 +47,6 @@ class TestRDocRIDriver < RDoc::TestCase
     ENV['RI_PAGER'] = pager_env
   end
 
-  def mu_pp(obj)
-    s = ''
-    s = PP.pp obj, s
-    s = s.force_encoding(Encoding.default_external) if defined? Encoding
-    s.chomp
-  end
-
   def test_self_dump
     util_store
 
@@ -249,6 +243,32 @@ class TestRDocRIDriver < RDoc::TestCase
     assert_equal expected, out
   end
 
+  def test_add_method_that_is_alias_for_original
+    util_store
+
+    out = doc
+
+    @driver.add_method out, 'Qux#aliased'
+
+    expected =
+      doc(
+        head(1, 'Qux#aliased'),
+        blank_line,
+        para('(from ~/.rdoc)'),
+        rule(1),
+        blank_line,
+        para('alias comment'),
+        blank_line,
+        blank_line,
+        para('(this method is alias for Qux#original)'),
+        blank_line,
+        para('original comment'),
+        blank_line,
+        blank_line)
+
+    assert_equal expected, out
+  end
+
   def test_add_method_attribute
     util_store
 
@@ -288,7 +308,7 @@ class TestRDocRIDriver < RDoc::TestCase
     assert_equal expected, out
   end
 
-  def test_add_method_overriden
+  def test_add_method_overridden
     util_multi_store
 
     out = doc
@@ -354,6 +374,22 @@ class TestRDocRIDriver < RDoc::TestCase
     assert_equal expected, out
   end
 
+  def test_output_width
+    @options[:width] = 10
+    driver = RDoc::RI::Driver.new @options
+
+    doc = @RM::Document.new
+    doc << @RM::IndentedParagraph.new(0, 'new, parse, foo, bar, baz')
+
+    out, = capture_io do
+      driver.display doc
+    end
+
+    expected = "new, parse, foo,\nbar, baz\n"
+
+    assert_equal expected, out
+  end
+
   def test_add_method_list_interative
     @options[:interactive] = true
     driver = RDoc::RI::Driver.new @options
@@ -396,6 +432,7 @@ class TestRDocRIDriver < RDoc::TestCase
       'Foo::Bar'  => [@store1],
       'Foo::Baz'  => [@store1, @store2],
       'Inc'       => [@store1],
+      'Qux'       => [@store1],
     }
 
     classes = @driver.classes
@@ -652,7 +689,7 @@ class TestRDocRIDriver < RDoc::TestCase
     assert_match %r%^=== Implementation from Foo%, out
   end
 
-  def test_display_method_overriden
+  def test_display_method_overridden
     util_multi_store
 
     out, = capture_io do
@@ -696,6 +733,14 @@ Foo::Bar#bother
     assert_equal expected, out
   end
 
+  def test_display_name_not_found_special
+    util_store
+
+    assert_raises RDoc::RI::Driver::NotFoundError do
+      assert_equal false, @driver.display_name('Set#[]')
+    end
+  end
+
   def test_display_method_params
     util_store
 
@@ -747,6 +792,25 @@ Foo::Bar#bother
     assert_match %r%README\.md%,                 out
   end
 
+  def test_display_page_extension
+    util_store
+
+    other = @store1.add_file 'README.EXT'
+    other.parser = RDoc::Parser::Simple
+    other.comment =
+      doc(
+        head(1, 'README.EXT'),
+        para('This is the other README'))
+
+    @store1.save_page other
+
+    out, = capture_io do
+      @driver.display_page 'home:README.EXT'
+    end
+
+    assert_match 'other README', out
+  end
+
   def test_display_page_ignore_directory
     util_store
 
@@ -813,6 +877,38 @@ Foo::Bar#bother
     end
   end
 
+  def test_expand_class_2
+    @store1 = RDoc::RI::Store.new @home_ri, :home
+
+    @top_level = @store1.add_file 'file.rb'
+
+    @cFoo = @top_level.add_class RDoc::NormalClass, 'Foo'
+    @mFox = @top_level.add_module RDoc::NormalModule, 'Fox'
+    @cFoo_Bar = @cFoo.add_class RDoc::NormalClass, 'Bar'
+    @store1.save
+
+    @driver.stores = [@store1]
+    assert_raises RDoc::RI::Driver::NotFoundError do
+      @driver.expand_class 'F'
+    end
+    assert_equal 'Foo::Bar',  @driver.expand_class('F::Bar')
+    assert_equal 'Foo::Bar',  @driver.expand_class('F::B')
+  end
+
+  def test_expand_class_3
+    @store1 = RDoc::RI::Store.new @home_ri, :home
+
+    @top_level = @store1.add_file 'file.rb'
+
+    @cFoo = @top_level.add_class RDoc::NormalClass, 'Foo'
+    @mFox = @top_level.add_module RDoc::NormalModule, 'FooBar'
+    @store1.save
+
+    @driver.stores = [@store1]
+
+    assert_equal 'Foo',  @driver.expand_class('Foo')
+  end
+
   def test_expand_name
     util_store
 
@@ -870,6 +966,7 @@ Foo::Bar#bother
       [@store1, 'Foo::Bar',  'Foo::Bar',  :both, 'blah'],
       [@store1, 'Foo::Baz',  'Foo::Baz',  :both, 'blah'],
       [@store1, 'Inc',       'Inc',       :both, 'blah'],
+      [@store1, 'Qux',       'Qux',       :both, 'blah'],
     ]
 
     assert_equal expected, items
@@ -916,11 +1013,35 @@ Foo::Bar#bother
     assert_equal 'nonexistent', e.name
   end
 
+  def test_did_you_mean
+    skip 'skip test with did_you_men' unless defined? DidYouMean::SpellChecker
+
+    util_ancestors_store
+
+    e = assert_raises RDoc::RI::Driver::NotFoundError do
+      @driver.lookup_method 'Foo.i_methdo'
+    end
+    assert_equal "Nothing known about Foo.i_methdo\nDid you mean?  i_method", e.message
+
+    e = assert_raises RDoc::RI::Driver::NotFoundError do
+      @driver.lookup_method 'Foo#i_methdo'
+    end
+    assert_equal "Nothing known about Foo#i_methdo\nDid you mean?  i_method", e.message
+
+    e = assert_raises RDoc::RI::Driver::NotFoundError do
+      @driver.lookup_method 'Foo::i_methdo'
+    end
+    assert_equal "Nothing known about Foo::i_methdo\nDid you mean?  c_method", e.message
+  end
+
   def test_formatter
     tty = Object.new
     def tty.tty?() true; end
 
-    driver = RDoc::RI::Driver.new
+    @options.delete :use_stdout
+    @options.delete :formatter
+
+    driver = RDoc::RI::Driver.new @options
 
     assert_instance_of @RM::ToAnsi, driver.formatter(tty)
 
@@ -979,7 +1100,7 @@ Foo::Bar#bother
       @driver.list_known_classes
     end
 
-    assert_equal "Ambiguous\nExt\nFoo\nFoo::Bar\nFoo::Baz\nInc\n", out
+    assert_equal "Ambiguous\nExt\nFoo\nFoo::Bar\nFoo::Baz\nInc\nQux\n", out
   end
 
   def test_list_known_classes_name
@@ -1050,7 +1171,7 @@ Foo::Bar#bother
     method = @driver.load_method(@store2, :instance_methods, 'Bar', '#',
                                  'inherit')
 
-    assert_equal nil, method
+    assert_nil method
   end
 
   def test_load_methods_matching
@@ -1075,6 +1196,17 @@ Foo::Bar#bother
     assert_equal expected, @driver.load_methods_matching('Bar#inherit')
   end
 
+  def test_load_method_missing
+    util_store
+
+    FileUtils.rm @store1.method_file 'Foo', '#inherit'
+
+    method = @driver.load_method(@store1, :instance_methods, 'Foo', '#',
+                                 'inherit')
+
+    assert_equal '(unknown)#inherit', method.full_name
+  end
+
   def _test_page # this test doesn't do anything anymore :(
     @driver.use_stdout = false
 
@@ -1144,33 +1276,41 @@ Foo::Bar#bother
 
     assert_equal 'ruby',   klass, 'ruby project'
     assert_equal ':',      type,  'ruby type'
-    assert_equal nil,      meth,  'ruby page'
+    assert_nil             meth,  'ruby page'
+  end
+
+  def test_parse_name_page_extenson
+    klass, type, meth = @driver.parse_name 'ruby:README.EXT'
+
+    assert_equal 'ruby',      klass, 'ruby project'
+    assert_equal ':',         type,  'ruby type'
+    assert_equal 'README.EXT', meth,  'ruby page'
   end
 
   def test_parse_name_single_class
     klass, type, meth = @driver.parse_name 'Foo'
 
     assert_equal 'Foo', klass, 'Foo class'
-    assert_equal nil,   type,  'Foo type'
-    assert_equal nil,   meth,  'Foo method'
+    assert_nil          type,  'Foo type'
+    assert_nil          meth,  'Foo method'
 
     klass, type, meth = @driver.parse_name 'Foo#'
 
     assert_equal 'Foo', klass, 'Foo# class'
     assert_equal '#',   type,  'Foo# type'
-    assert_equal nil,   meth,  'Foo# method'
+    assert_nil          meth,  'Foo# method'
 
     klass, type, meth = @driver.parse_name 'Foo::'
 
     assert_equal 'Foo', klass, 'Foo:: class'
     assert_equal '::',  type,  'Foo:: type'
-    assert_equal nil,   meth,  'Foo:: method'
+    assert_nil          meth,  'Foo:: method'
 
     klass, type, meth = @driver.parse_name 'Foo.'
 
     assert_equal 'Foo', klass, 'Foo. class'
     assert_equal '.',   type,  'Foo. type'
-    assert_equal nil,   meth,  'Foo. method'
+    assert_nil          meth,  'Foo. method'
 
     klass, type, meth = @driver.parse_name 'Foo#Bar'
 
@@ -1195,14 +1335,14 @@ Foo::Bar#bother
     klass, type, meth = @driver.parse_name 'Foo::Bar'
 
     assert_equal 'Foo::Bar', klass, 'Foo::Bar class'
-    assert_equal nil,        type,  'Foo::Bar type'
-    assert_equal nil,        meth,  'Foo::Bar method'
+    assert_nil               type,  'Foo::Bar type'
+    assert_nil               meth,  'Foo::Bar method'
 
     klass, type, meth = @driver.parse_name 'Foo::Bar#'
 
     assert_equal 'Foo::Bar', klass, 'Foo::Bar# class'
     assert_equal '#',        type,  'Foo::Bar# type'
-    assert_equal nil,        meth,  'Foo::Bar# method'
+    assert_nil               meth,  'Foo::Bar# method'
 
     klass, type, meth = @driver.parse_name 'Foo::Bar#baz'
 
@@ -1380,10 +1520,19 @@ Foo::Bar#bother
     @inherit = @cFoo.add_method RDoc::AnyMethod.new(nil, 'inherit')
     @inherit.record_location @top_level
 
-    # overriden by Bar in multi_store
-    @overriden = @cFoo.add_method RDoc::AnyMethod.new(nil, 'override')
-    @overriden.comment = 'must not be displayed in Bar#override'
-    @overriden.record_location @top_level
+    # overridden by Bar in multi_store
+    @overridden = @cFoo.add_method RDoc::AnyMethod.new(nil, 'override')
+    @overridden.comment = 'must not be displayed in Bar#override'
+    @overridden.record_location @top_level
+
+    @cQux = @top_level.add_class RDoc::NormalClass, 'Qux'
+
+    @original = @cQux.add_method RDoc::AnyMethod.new(nil, 'original')
+    @original.comment = 'original comment'
+    @original.record_location @top_level
+
+    @aliased = @original.add_alias RDoc::Alias.new(nil, 'original', 'aliased', 'alias comment'), @cQux
+    @aliased.record_location @top_level
 
     @store1.save
 
@@ -1391,4 +1540,3 @@ Foo::Bar#bother
   end
 
 end
-
